import React, { useMemo } from 'react'

// DnD: https://docs.dndkit.com/
import { SortableContext, useSortable } from '@dnd-kit/sortable'
import { useDndContext } from '@dnd-kit/core'
import { CSS } from '@dnd-kit/utilities'

// via-ui
import { Card, CardContent, CardHeader } from '../ui/card'
import { ScrollArea, ScrollBar } from '../ui/scroll-area'

// Other
import { TaskCard } from '../TaskCard'
import { cva } from 'class-variance-authority'
import { cn } from '../../lib/utils'
import { BoardColumnProps, ColumnDragData } from './boardColumn.types'
import { useScreenType } from '../../hooks'
import { Button } from '../ui/button'
import { Icons } from '../icons'
import Tooltip from 'via-ui/tooltip'

export const BoardColumn = ({ column, tasks, isOverlay }: BoardColumnProps) => {
  const screenType = useScreenType()

  const tasksIds = useMemo(() => {
    return tasks.map((task) => task.id)
  }, [tasks])

  const {
    setNodeRef,
    attributes,
    listeners,
    transform,
    transition,
    isDragging
  } = useSortable({
    id: column.id,
    data: {
      type: 'Column',
      column
    } satisfies ColumnDragData,
    attributes: {
      roleDescription: `Column: ${column.title}`
    }
  })

  const style = {
    transition,
    transform: CSS.Translate.toString(transform)
  }

  const variants = cva(
    'group/board-column flex shrink-0 cursor-pointer select-none snap-center flex-col rounded-lg bg-gray-100 shadow-md hover:bg-gray-200',
    {
      variants: {
        dragging: {
          default: 'border-2 border-transparent',
          over: 'border-4 border-dashed border-gray-400 opacity-30',
          overlay: 'rotate-2 ring-2 ring-black'
        },
        screenType: {
          desktop: 'h-[500px] max-h-[500px] w-[300px] max-w-full',
          mobile: 'h-[75vh] w-full min-w-full pb-2'
        }
      }
    }
  )

  const variantsTask = cva(
    'flex grow gap-2 p-2', {
      variants: {
        screenType: {
          desktop: 'flex-col',
          mobile: 'flex-col'
        }
      }
    })

  return (
    <Card
      ref={setNodeRef}
      style={style}
      className={variants({
        dragging: isOverlay ? 'overlay' : isDragging ? 'over' : undefined,
        screenType: screenType.isMobile ? 'mobile' : 'desktop'
      })}
    >
      <CardHeader
        className={cn(
          'relative space-between flex flex-row items-center border-b-2 p-2 text-left font-semibold rounded-t-lg text-white',
          column.color ? column.color : 'bg-gray-500'
        )}
        {...attributes}
        {...listeners}
      >
        <span className='mr-auto'>
          {column.title}
        </span>
        <Tooltip className='hidden group-hover/board-column:block' placement='top' content='Add task'>
          <Button className='group/plus-circle -my-1 flex h-auto w-auto cursor-pointer items-center justify-center bg-transparent p-0.5 hover:bg-gray-200 group-hover/board-column:block'>
            <Icons.plusCircle className='h-5 w-5 group-hover/plus-circle:text-black'/>
          </Button>
        </Tooltip>
      </CardHeader>

      <ScrollArea>
        <CardContent className={variantsTask({
          screenType: screenType.isMobile ? 'mobile' : 'desktop'
        })}>
          <SortableContext items={tasksIds}>
            {tasks.map((task) => (
              <TaskCard key={task.id} task={task} />
            ))}
          </SortableContext>
        </CardContent>
        <ScrollBar orientation={screenType.isMobile ? 'horizontal' : 'vertical'} />
      </ScrollArea>

    </Card>
  )
}

export function BoardContainer ({ children }: { children: React.ReactNode }) {
  const dndContext = useDndContext()
  const screenType = useScreenType()

  const variations = cva('flex md:px-0 lg:justify-center', {
    variants: {
      dragging: {
        default: 'snap-x snap-mandatory',
        active: 'snap-none'
      }
    }
  })

  const variationsColumn = cva('flex w-full max-w-full items-start justify-start gap-4', {
    variants: {
      screenType: {
        desktop: 'flex-row',
        mobile: 'flex-row overflow-x-auto'
      }
    }
  })

  return (
    screenType.isMobile
      ? (
        <div className={variationsColumn({ screenType: 'mobile' })}>
          {children}
        </div>
      )
      : (
        <ScrollArea
          className={variations({ dragging: dndContext.active ? 'active' : 'default' })}
        >
          <div className={variationsColumn({ screenType: 'desktop' })}>
            {children}
          </div>
          <ScrollBar orientation="horizontal" />
        </ScrollArea>
      )
  )
}
