import React, { useMemo, useRef, useState } from 'react'
import { createPortal } from 'react-dom'

// DndKit:
import {
  DndContext,
  type DragEndEvent,
  type DragOverEvent,
  DragOverlay,
  type DragStartEvent,
  useSensor,
  useSensors,
  KeyboardSensor,
  Announcements,
  UniqueIdentifier,
  TouchSensor,
  MouseSensor
} from '@dnd-kit/core'
import { SortableContext, arrayMove } from '@dnd-kit/sortable'
import { TaskCard } from '../TaskCard'

// Others
import { BoardColumn, BoardContainer } from '../BoardColumn'
import { hasDraggableData } from '../../utils'
import { coordinateGetter } from '../../multipleContainersKeyboardPreset'
import { useIsClient, useScreenType } from '../../hooks'
import { initialTasks, ColumnId, defaultCols } from '../../constants/defaults'
import { toast } from 'react-toastify'
import { Column } from '../BoardColumn/boardColumn.types'
import { Task } from '../TaskCard/taskCard.types'
import { KanbanBoardProps } from './kanbanBoard.type'
import SearchHeader from '../SearchHeader'
import { Card, CardHeader } from '../ui/card'
import { cva } from 'class-variance-authority'
import { Icons } from '../icons'
import { cn } from '../../lib/utils'
// import { Button } from '../ui/button'

const defaultSearchHeader = {
  hide: false,
  upperButtons: null,
  hideInput: false,
  inputSearchStyles: {},
  inputSearchClassName: '',
  inputPlaceholder: 'Buscar...',
  upperRightButtons: null,
  newButtonOnClick: () => {},
  newButtonStyles: {},
  showNewButton: true
}

export const KanbanBoard = ({
  columns = defaultCols,
  tasks = initialTasks,
  i18n = { t: () => { return 'txtNone' } },
  searchHeader = defaultSearchHeader
}: KanbanBoardProps) => {
  const screenType = useScreenType()

  const [_columns, setColumns] = useState<Column[]>(columns)
  const pickedUpTaskColumn = useRef<ColumnId | null>(null)
  const columnsId = useMemo(() => _columns.map((col) => col.id), [_columns])

  const [_tasks, setTasks] = useState<Task[]>(tasks)

  const [activeColumn, setActiveColumn] = useState<Column | null>(null)

  const [activeTask, setActiveTask] = useState<Task | null>(null)

  const isClient = useIsClient()

  const sensors = useSensors(
    useSensor(MouseSensor, {
      activationConstraint: {
        delay: 250,
        tolerance: 5
      }
    }),
    useSensor(TouchSensor, {
      activationConstraint: {
        delay: 250,
        tolerance: 5
      }
    }),
    useSensor(KeyboardSensor, {
      coordinateGetter
    })
  )

  function getDraggingTaskData (taskId: UniqueIdentifier, columnId: ColumnId) {
    const tasksInColumn = _tasks.filter((task) => task.columnId === columnId)
    const taskPosition = tasksInColumn.findIndex((task) => task.id === taskId)
    const column = _columns.find((col) => col.id === columnId)
    return {
      tasksInColumn,
      taskPosition,
      column
    }
  }

  function onDragStart (event: DragStartEvent) {
    if (!hasDraggableData(event.active)) return
    const data = event.active.data.current
    if (data?.type === 'Column') {
      setActiveColumn(data.column)
      return
    }

    if (data?.type === 'Task') {
      setActiveTask(data.task)
    }
  }

  function onDragOver (event: DragOverEvent) {
    const { active, over } = event
    if (!over) return

    const activeId = active.id
    const overId = over.id

    if (activeId === overId) return

    if (!hasDraggableData(active) || !hasDraggableData(over)) return

    const activeData = active.data.current
    const overData = over.data.current

    const isActiveATask = activeData?.type === 'Task'
    const isOverATask = activeData?.type === 'Task'

    if (!isActiveATask) return

    // Im dropping a Task over another Task
    if (isActiveATask && isOverATask) {
      setTasks((tasks) => {
        const activeIndex = tasks.findIndex((t) => t.id === activeId)
        const overIndex = tasks.findIndex((t) => t.id === overId)
        const activeTask = tasks[activeIndex]
        const overTask = tasks[overIndex]
        if (
          activeTask &&
          overTask &&
          activeTask.columnId !== overTask.columnId
        ) {
          activeTask.columnId = overTask.columnId
          return arrayMove(tasks, activeIndex, overIndex - 1)
        }

        return arrayMove(tasks, activeIndex, overIndex)
      })
    }

    const isOverAColumn = overData?.type === 'Column'

    // Im dropping a Task over a column
    if (isActiveATask && isOverAColumn) {
      setTasks((tasks) => {
        const activeIndex = tasks.findIndex((t) => t.id === activeId)
        const activeTask = tasks[activeIndex]
        if (activeTask) {
          activeTask.columnId = overId as ColumnId
          return arrayMove(tasks, activeIndex, activeIndex)
        }
        return tasks
      })
    }
  }

  function onDragEnd (event: DragEndEvent) {
    setActiveColumn(null)
    setActiveTask(null)

    const { active, over } = event
    if (!over) return

    const activeId = active.id
    const overId = over.id

    if (!hasDraggableData(active)) return

    const activeData = active.data.current

    if (activeId === overId) return

    const isActiveAColumn = activeData?.type === 'Column'
    if (!isActiveAColumn) return

    setColumns((columns) => {
      const activeColumnIndex = columns.findIndex((col) => col.id === activeId)

      const overColumnIndex = columns.findIndex((col) => col.id === overId)

      return arrayMove(columns, activeColumnIndex, overColumnIndex)
    })
  }

  const announcements: Announcements = {
    onDragStart ({ active }) {
      if (!hasDraggableData(active)) return
      if (active.data.current?.type === 'Column') {
        const startColumnIdx = columnsId.findIndex((id) => id === active.id)
        const startColumn = _columns[startColumnIdx]
        return `Picked up Column ${startColumn?.title} at position: ${
          startColumnIdx + 1
        } of ${columnsId.length}`
      } else if (active.data.current?.type === 'Task') {
        pickedUpTaskColumn.current = active.data.current.task.columnId
        const { tasksInColumn, taskPosition, column } = getDraggingTaskData(
          active.id,
          pickedUpTaskColumn.current
        )
        return `Picked up Task ${
          active.data.current.task.content
        } at position: ${taskPosition + 1} of ${
          tasksInColumn.length
        } in column ${column?.title}`
      }
    },
    onDragOver ({ active, over }) {
      if (!hasDraggableData(active) || !hasDraggableData(over)) return

      if (
        active.data.current?.type === 'Column' &&
        over.data.current?.type === 'Column'
      ) {
        const overColumnIdx = columnsId.findIndex((id) => id === over.id)
        return `Column ${active.data.current.column.title} was moved over ${
          over.data.current.column.title
        } at position ${overColumnIdx + 1} of ${columnsId.length}`
      } else if (
        active.data.current?.type === 'Task' &&
        over.data.current?.type === 'Task'
      ) {
        const { tasksInColumn, taskPosition, column } = getDraggingTaskData(
          over.id,
          over.data.current.task.columnId
        )
        if (over.data.current.task.columnId !== pickedUpTaskColumn.current) {
          return `Task ${
            active.data.current.task.content
          } was moved over column ${column?.title} in position ${
            taskPosition + 1
          } of ${tasksInColumn.length}`
        }
        return `Task was moved over position ${taskPosition + 1} of ${
          tasksInColumn.length
        } in column ${column?.title}`
      }
    },
    onDragEnd ({ active, over }) {
      toast.success(`${active.data.current?.type} dropped`)

      // Resetear a estado anterior

      if (!hasDraggableData(active) || !hasDraggableData(over)) {
        pickedUpTaskColumn.current = null
        return
      }
      if (
        active.data.current?.type === 'Column' &&
        over.data.current?.type === 'Column'
      ) {
        const overColumnPosition = columnsId.findIndex((id) => id === over.id)

        return `Column ${
          active.data.current.column.title
        } was dropped into position ${overColumnPosition + 1} of ${
          columnsId.length
        }`
      } else if (
        active.data.current?.type === 'Task' &&
        over.data.current?.type === 'Task'
      ) {
        const { tasksInColumn, taskPosition, column } = getDraggingTaskData(
          over.id,
          over.data.current.task.columnId
        )
        if (over.data.current.task.columnId !== pickedUpTaskColumn.current) {
          return `Task was dropped into column ${column?.title} in position ${
            taskPosition + 1
          } of ${tasksInColumn.length}`
        }
        return `Task was dropped into position ${taskPosition + 1} of ${
          tasksInColumn.length
        } in column ${column?.title}`
      }
      pickedUpTaskColumn.current = null
    },
    onDragCancel ({ active }) {
      pickedUpTaskColumn.current = null
      if (!hasDraggableData(active)) return
      return `Dragging ${active.data.current?.type} cancelled.`
    }
  }

  const variants = cva(
    'group flex shrink-0 cursor-pointer snap-center flex-col rounded-lg border-none bg-gray-50/90 shadow-md hover:bg-gray-100',
    {
      variants: {
        screenType: {
          desktop: 'h-[500px] max-h-[500px] w-[300px] max-w-full',
          mobile: 'max-h-auto h-auto w-full min-w-full'
        }
      }
    }
  )

  return (
    <div className='rounded-lg bg-white p-2 shadow-md'>

      { !searchHeader.hide && <SearchHeader i18n={i18n} searchHeader={searchHeader} />}

      <DndContext
        accessibility={{
          announcements
        }}
        sensors={sensors}
        onDragStart={onDragStart}
        onDragOver={onDragOver}
        onDragEnd={onDragEnd}
      >
        <BoardContainer>
          <SortableContext items={columnsId}>
            {_columns.map((col) => (
              <BoardColumn
                key={col.id}
                column={col}
                tasks={_tasks.filter((task) => task.columnId === col.id)}
              />
            ))}
            <Card className={variants({ screenType: screenType.isMobile ? 'mobile' : 'desktop' })} onClick={() => {}}>
              <CardHeader className={cn('space-between flex select-none flex-row items-center justify-center border-b-2 bg-slate-300 p-2 font-semibold text-gray-500',
                screenType.isMobile ? 'rounded-lg' : 'rounded-t-lg'
              )}>
                <span className='flex items-center gap-1 whitespace-nowrap'>
                  <Icons.plusCircle className='h-5 w-5'/>
                  Add section
                </span>
              </CardHeader>
            </Card>
          </SortableContext>
        </BoardContainer>

        {isClient && 'document' in window &&
        createPortal(
          <DragOverlay>
            {activeColumn && (
              <BoardColumn
                isOverlay
                column={activeColumn}
                tasks={_tasks.filter(
                  (task) => task.columnId === activeColumn.id
                )}
              />
            )}
            {activeTask && <TaskCard task={activeTask} isOverlay />}
          </DragOverlay>,
          document.body
        )}
      </DndContext>
    </div>
  )
}
