import { useEffect, useState } from 'react'

/**
 * Hook de React para determinar si el entorno actual es del lado del cliente.
 *
 * En el contexto de aplicaciones de React, especialmente aquellas que utilizan
 * renderización en el lado del servidor (SSR), es crucial identificar si el
 * código se está ejecutando en el entorno del cliente (navegador) o del servidor.
 * Este hook proporciona una manera confiable de determinar este contexto,
 * permitiendo ejecutar código específico del lado del cliente de manera segura.
 *
 * El hook utiliza `useState` para establecer el estado inicial basado en la
 * disponibilidad del objeto `window`, un indicativo claro del entorno del cliente.
 * A través de `useEffect`, ajusta este estado solo cuando el componente se monta
 * en el navegador, garantizando una detección precisa y evitando ejecuciones
 * innecesarias en el lado del servidor.
 *
 * @returns {boolean} Retorna `true` si el código se está ejecutando en el entorno del cliente.
 *                    De lo contrario, retorna `false`.
 *
 * @example
 * import React from 'react';
 * import useIsClient from './useIsClient';
 *
 * const MyComponent: React.FC = () => {
 *     const isClient = useIsClient();
 *
 *     if (isClient) {
 *         // Ejemplo: Código seguro para ejecutar en el lado del cliente
 *         console.log('Ancho de la ventana:', window.innerWidth);
 *     }
 *
 *     return <div>Contenido del componente</div>;
 * };
 *
 * export default MyComponent;
 */
export const useIsClient = (): boolean => {
  const [isClient, setIsClient] = useState(typeof window === 'object')

  useEffect(() => {
    // Este efecto no se ejecutará en el servidor, solo se activa en el entorno del cliente.
    if (!isClient) {
      setIsClient(true)
    }
  }, [isClient])

  return isClient
}
